/*
 * rx.c
 *
 *  Created on: Dec 9, 2017
 *      Author: rafal
 */

#include "rx.h"

uint8_t keys[] = {'u', 'p', 'd', 's'};
volatile int32_t data[KEY] = {200, 0, 0, 4500};

enum state {WAIT_STATE, RECEIVE_STATE, NUMBER_STATE};

void rx(uint8_t c){
	static enum state s = WAIT_STATE;
	static uint8_t key = 0;
	static int32_t number = 0;
	static int8_t sign = 1;
	//printf("State i: %d\t key: %d\t number: %d\t sign: %d\n", s, key, number, sign);
	switch(s){
		case WAIT_STATE:
			for(key = 0; key < KEY && c != keys[key]; key++);
			if(key < KEY){
				s = RECEIVE_STATE;
				number = 0;
				sign = 1;
			}
			break;
		case RECEIVE_STATE:
			if (c == '-') {
				s = NUMBER_STATE;
				sign = -1;
				break;
			} else if (c >= '0' && c <= '9') {
				s = NUMBER_STATE;
				number = c - '0';
			} else
				s = WAIT_STATE;
			break;
		case NUMBER_STATE:
			if (c >= '0' && c <= '9'){
				number *= 10;
				number += c - '0';
			} else if(c == '\n'){
				//printf("SAVE\n");
				s = WAIT_STATE;
				data[key] = sign * number;
				//printf("data: %d\n", data[key]);
			} else
				s = WAIT_STATE;
			break;
	}
	//printf("State o: %d\t key: %d\t number: %d\t sign: %d\t c: %d\n", s, key, number, sign, c);
}

#ifndef TINYTEST_NOTESTING

int32_t test(uint32_t L, uint8_t input[], int32_t out[4][KEY]){
	int32_t errors = 0;
	for(int i=0; i< L; i++){
		//printf("%d %c\n", i, input[i]);
		rx(input[i]);
		for(int j=0; j<KEY; j++){
				//printf("%d - %d\n", out[i][j], data[j]);
				if(out[i][j] != data[j])
					errors++;
			}
	}
	printf("Err: %d\n", errors);
	return errors;
}

uint8_t in[] = {'p', '1', '0', '\n', 'p', '-', '5', '2', '\n', 'p', '1', 'a', 'u', '-', '4', '2', '1', '\n', 'c', 'a', '\n'};
int32_t out[4][KEY] = {{0,0,0,0}, {0,0,0,0}, {0,0,0,0}, {0,10,0,0}, {0,10,0,0}, {0,10,0,0}, {0,10,0,0}, {0,10,0,0}, {0,-52,0,0}, {0,-52,0,0}, {0,-52,0,0}, {0,-52,0,0}, {0,-52,0,0}, {0,-52,0,0}, {0,-52,0,0}, {0,-52,0,0}, {0,-52,0,0}, {0,-52,-421,0}, {0,-52,-421,0}, {0,-52,-421,0}, {0,-52,-421,0}};

int32_t t1(){
	int32_t e = test(4, in, out);
	printf("s%d\n", e);
	TINYTEST_EQUAL(0, e);
}

TINYTEST_START_SUITE(Rx);
  TINYTEST_ADD_TEST(t1,NULL,NULL);
TINYTEST_END_SUITE();

#endif


