import serial
import json


class MagLev:
    BAUD = 2000000

    def __init__(self):
        self.received = ""
        self.com = None
        self.ctrl = {}
        self.skip = 10
        self.state = {"com_name": None,
                      "message": "Not connected",
                      "last_data": None
                      }

    def open_com(self, name):
        if self.com is None:
            try:
                self.com = serial.Serial(name, self.BAUD, timeout=0)
                self.com.flushInput()
                self.state["com_name"] = name
                self.state["message"] = "Port opened"
            except serial.serialutil.SerialException:
                self.state["message"] = "Failed to open"

    def close_com(self):
        if self.com is not None:
            self.com.close()
            self.state["com_name"] = None
            self.com = None

    def set_ctrl(self, ctrl):
        self.ctrl = ctrl
        c = self._to_send("p", "p")
        c += self._to_send("d", "d")
        c += self._to_send("sp", "s")
        c += self._to_send("u0", "u", 1)
        if self.com is not None:
            try:
                self.com.write(c)
                self.state["message"] = "Data send"
            except serial.serialutil.SerialException:
                self.com.close()
                self.com = None
                self.state["com_name"] = None
                self.state["message"] = "COM write error"

    def _to_send(self, name, command, multiplier=1000):
        if name not in self.ctrl:
            return b""
        n = int(float(self.ctrl[name]) * multiplier)
        n = "{}{}\n".format(command, n)
        return bytes(n, "ascii")

    def get_state(self):
        self.state["last_data"] = self._read_data()
        state_json = json.dumps(self.state)
        return state_json

    def _read_data(self):
        if self.com is None:
            return None
        dc = self.com.inWaiting()
        self.received += self.com.read(dc).decode("utf-8")
        data = self.received.split("\r\n")
        if data[-1] != "":
            self.received = data[-1]
        else:
            self.received = ""
        # remove last element (empty or not ready)
        # get only evry 10th element
        data = data[:-1:self.skip]
        data = [d.split("\t") for d in data]
        time = []
        pos = []
        sp = []
        u = []
        val = []
        for d in data:
            time.append(float(d[0])/1000)
            pos.append(float(d[3])/1000)
            sp.append(float(d[1])/1000)
            u.append(int(d[5]))
            val.append(int(d[2]))
        data = {"time": time, "pos": pos, "sp": sp, "u": u, "val": val}
        return data
