(function() { // Begin scoping function
var chartColors = {
    red: 'rgb(255, 99, 132)',
    orange: 'rgb(255, 159, 64)',
    yellow: 'rgb(255, 205, 86)',
    green: 'rgb(75, 192, 192)',
    blue: 'rgb(54, 162, 235)',
    purple: 'rgb(153, 102, 255)',
    grey: 'rgb(201, 203, 207)'
};

var adc_value = [];

var pos_data = {
    labels: [],
    datasets: [{
        label: "Position, mm",
        backgroundColor: chartColors.red,
        borderColor: chartColors.red,
        fill: false,
        pointRadius: 0,
        data: [],
    },{
        label: "Set position, mm",
        backgroundColor: chartColors.blue,
        borderColor: chartColors.blue,
        fill: false,
        pointRadius: 0,
        data: [],
    }]
};

var u_data = {
    labels: [],
    datasets: [{
        label: "Control, ‰",
        backgroundColor: chartColors.orange,
        borderColor: chartColors.orange,
        fill: false,
        pointRadius: 0,
        data: [],
    }]
};

var options = {
    elements: {
        line: {
            tension: 0, // disables bezier curves
        },
    },
    scales: {
            xAxes: [{
                    display: true,
                    scaleLabel: {
                        display: true,
                        labelString: 'Time, s'
                    }
            }],
            yAxes: [{
                    display: true,
                    scaleLabel: {
                        display: true,
                        labelString: ''
                    }
            }]
    },
    animation: {
        duration: 0.2, // general animation time
    },
    hover: {
        animationDuration: 0, // duration of animations when hovering an item
    },
    responsiveAnimationDuration: 0, // animation duration after a resize
};

var history = 500;

var pos_canvas = document.getElementById("position").getContext('2d');
var pos_chart = new Chart(pos_canvas, {
    type: 'line',
    data: pos_data,
    options: options
});

var u_canvas = document.getElementById("u").getContext('2d');
var u_chart = new Chart(u_canvas, {
    type: 'line',
    data: u_data,
    options: options
});

setInterval(function(){
    $.getJSON("get_state", "", function(result){
        message = ""
        if(result["com_name"])
            message = "Open " + result["com_name"] + "<hr />";
        else
            message = "Port closed<hr />";
        message += result["message"];
        $("#com_state").html(message);
        if(result["last_data"]){
            merge(pos_data["labels"], result["last_data"]["time"]);
            merge(pos_data["datasets"][0]["data"], result["last_data"]["pos"]);
            merge(pos_data["datasets"][1]["data"], result["last_data"]["sp"]);
            merge(u_data["labels"], result["last_data"]["time"]);
            merge(u_data["datasets"][0]["data"], result["last_data"]["u"]);
            merge(adc_value, result["last_data"]["val"]);
            pos_chart.update();
            u_chart.update();
        }
    });
}, 100);

function merge(a1, a2){
    Array.prototype.push.apply(a1, a2);
    a1.splice(0, Math.max(a1.length - history, 0));
}

$("#ctrl_set").click(function(){
    $.post("ctrl_set",
           {
                sp: $("#sp").val(),
                u0: $("#u0").val(),
                p:  $("#p").val(),
                d:  $("#d").val()
           },
           function(result){});
});

$("#com_open").click(function(){
    $.post("com",
           {name: $("#com_name").val()},
           function(result){});
});

$("#chart_apply").click(function(){
    pps = parseFloat($("#chart_pps").val());
    ht = parseFloat($("#chart_ht").val());
    skip = parseInt(1000/pps);
    history = parseInt(ht*pps);
    $.post("set_skip",
           {skip: skip},
           function(result){});
});

var calibration_data = {
    datasets: [{
        label: "data point",
        backgroundColor: chartColors.orange,
        borderColor: chartColors.orange,
        fill: false,
        pointRadius: 3,
        data: [],
    }]
};

var calibration_options = {
    elements: {
        line: {
            tension: 0, // disables bezier curves
        },
    },
    scales: {
            xAxes: [{
                    display: true,
                    type: 'linear',
                    scaleLabel: {
                        display: true,
                        labelString: 'ADC value'
                    }
            }],
            yAxes: [{
                    display: true,
                    type: 'linear',
                    scaleLabel: {
                        display: true,
                        labelString: 'distance, mm'
                    }
            }]
    },
    animation: {
        duration: 0.5, // general animation time
    },
    hover: {
        animationDuration: 0, // duration of animations when hovering an item
    },
    responsiveAnimationDuration: 0, // animation duration after a resize
};

var calibration_canvas = document.getElementById("calibration_chart").getContext('2d');
var calibration_chart = new Chart(calibration_canvas, {
    type: 'line',
    data: calibration_data,
    options: calibration_options
});

table_row = "<tr><td>Jacob</td><td>Thornton</td><td>@fat</td></tr>";

$("#measure").click(function(){
    distance = parseFloat($("#distance").val());
    value = 0;
    value_list = adc_value;
    value_list_len = value_list.length;
    for(i = 0; i < value_list_len; i++)
        value += value_list[i];
    value = value/value_list_len;

    data_set = calibration_data.datasets[0].data;
    var i;
    var len = data_set.length;
    for(i = 0; i < len; i++)
        if(data_set[i].y == distance)
            break;
    if(i < len){
        data_set[i].x = value;
    } else{
        data_set.push({x: value, y: distance});
    }

    data_set.sort(function(a, b){return a.x-b.x});

    generate_table(data_set);
    calibration_chart.update();
});

function generate_table(list){
    table = ""

    n = "int32_t N = " + list.length + ";<br />"
    x = "int32_t X[] = {";
    y = "int32_t Y[] = {";
    for(i = 0, len = list.length; i < len; i++) {
        table += "<tr><td>" + list[i].x + "</td><td>" +
            list[i].y + "</td><td><button type='button' id='" + i +
            "' class='btn btn-outline-danger calibration_delete'>✂</button></td></tr>";
        x += Math.round(list[i].x);
        y += list[i].y*1000;
        if(i < len-1) {
            x += ", ";
            y += ", ";
        }
    }
    x += "};<br />";
    y += "};";
    $("#calibration_table").html(table);
    $("#calibration_code").html(n + x + y);
}

$("#calibration_table").on('click','button', function (evt) {
    data_set = calibration_data.datasets[0].data;
    i = $(this).attr("id");
    data_set.splice(i, 1);
    generate_table(data_set);
    calibration_chart.update();
});

})();         // End scoping function